/*
General Graphics Functions
*/
#include <cybiko.h>

void snapshot(char *);

int getinput(struct module_t *ptr_main_module)
{
struct Message* ptr_message;
struct KeyParam* ptr_key_param;
int num=0;

ptr_message = cWinApp_get_message( ptr_main_module->m_process, 1, 1,MSG_USER );
	
if(ptr_message->msgid==MSG_KEYDOWN)
{
ptr_key_param = Message_get_key_param( ptr_message );

num=ptr_key_param->scancode;
}    

Message_delete(ptr_message);

return num;
}

int waitinput(struct module_t *ptr_main_module)
{
struct Message* ptr_message;
struct KeyParam* ptr_key_param;
int num=0,exit=0;

// Clear the message buffer of key downs presses
while(ptr_message = cWinApp_peek_message( ptr_main_module->m_process, TRUE, MSG_KEYDOWN, MSG_KEYDOWN ))
{
    Message_delete( ptr_message );
//	TRACE("Deleting message");
}

do
{
	ptr_message = cWinApp_get_message( ptr_main_module->m_process, 0, 1,MSG_USER );
		
	if(ptr_message->msgid==MSG_KEYDOWN)
	{
	  ptr_key_param = Message_get_key_param( ptr_message );  
	  num=ptr_key_param->scancode;
	  exit=1;
	}    
	
	Message_delete(ptr_message);
}while(exit==0);

if (num==KEY_S)
{
	snapshot(DisplayGraphics_get_page_ptr(ptr_main_module->m_gfx, 0));
}

return num;
}



//Converts an 8bit row to a 2bit (4ppb) row
void conv_row(char *out,char *in,long width,long j)
{
long apos=0,px=0;
int aw,i;
char p1,p2,p3,p4;

aw=(width & 0xfffc)>>2;
apos=(aw*(j-1));

for(i=0;i<aw;i++)
	{
	p1=3-((in[px++] & 0xff)>>6);
	p2=3-((in[px++] & 0xff)>>6);
	p3=3-((in[px++] & 0xff)>>6);
	p4=3-((in[px++] & 0xff)>>6);

	out[apos++]=(p1<<6)+(p2<<4)+(p3<<2)+p4;
	}
}

/*
Converts 24bit to 2bit
*/

char gbyte24(char r,char g,char b)
{
long av;

av=(r & 0xff)+(g & 0xff)+(b & 0xff);

av=(av/3)>>6;

return (3-av);
}

/*
Converts 24bit to 8bit
*/

char grey24(char r,char g,char b)
{
long av;

av=(r & 0xff)+(g & 0xff)+(b & 0xff);

return (av/3);
}


/*
Converts 16bit to 2bit
*/

char gbyte16(short col)
{
char r,g,b,av;

r=((col & 0x7c00)>>10);
g=((col & 992)>>5);
b=(col & 31);

av=((r+g+b)/3)>>3;

return (3-av);
}


/*
Plots a pixel directly to the screen
*/


void fplot(char *scr,long x,long y,long w,char c)
{
long ax,bx,aw,ay;
char byt,clr;

ax=(x & 0xfffc)>>2;
aw=(w & 0xfffc)>>2;

ay=y*aw;

//3- Flip order (<<2 = *4)
bx=3-(x-(ax<<2));

byt=c<<(bx<<1);
clr=0x03<<(bx<<1);

scr[ax+ay]=(scr[ax+ay]&~clr)+byt;
}

/*
Gauge
Shown at the bottom so we can safetly use the top of the screen as a buffer!
*/


void gauge(struct module_t *ptr_main_module,long cur,long mx)
{
long stp;
char *temp;
temp=(char *)calloc(20,sizeof(char));

stp=(cur*100)/mx;

sprintf(temp,"Loading... %ld%%",stp);

TGraph_set_color(ptr_main_module->m_gfx, CLR_BLACK );
TGraph_fill_rect(ptr_main_module->m_gfx,0,90,160,10);

TGraph_set_color(ptr_main_module->m_gfx, CLR_WHITE);
DisplayGraphics_draw_text(ptr_main_module->m_gfx,temp, 1, 90 );

DisplayGraphics_show(ptr_main_module->m_gfx);
free(temp);
}

/*
Screen Zoom 80x50 -> 160x100 
*/


void zoom(char *in,char *out,int width,int height,int ox,int oy)
{
int i,j,row=0,cnt=0,pos=0;
char a,b,c,d;

pos=((width>>2)*oy)+ox;

for(j=0;j<50;j++)
{
	for(i=0;i<20;i++)
	{
	// one byte in
	a=(in[pos+i] & 0xc0)>>6;
	b=(in[pos+i] & 0x30)>>4;
	c=(in[pos+i] & 0x0c)>>2;
	d=(in[pos+i] & 0x03);

	//two bytes out
	out[cnt++]=(a<<6)|(a<<4)|(b<<2)|b;
	out[cnt++]=(c<<6)|(c<<4)|(d<<2)|d;
	}

	//copy line down one
	memcpy(out+row+40,out+row,40);
	row+=80;
	
	// move down one line (display)
	cnt+=40;
	
	// move down one line
	pos+=(width>>2);
}
}


void center(struct module_t *ptr_main_module,char *text,int y)
{
int width,pos;

width=Font_string_width( DisplayGraphics_get_font( ptr_main_module->m_gfx),text );

pos=(160-width)>>1;	    
if (pos<0){pos=0;}

TGraph_set_color(ptr_main_module->m_gfx, CLR_WHITE );
DisplayGraphics_draw_text(ptr_main_module->m_gfx,text, pos+1, y+1 );
DisplayGraphics_draw_text(ptr_main_module->m_gfx,text, pos-1, y+1 );
DisplayGraphics_draw_text(ptr_main_module->m_gfx,text, pos+1, y-1 );
DisplayGraphics_draw_text(ptr_main_module->m_gfx,text, pos-1, y-1 );

TGraph_set_color(ptr_main_module->m_gfx, CLR_BLACK );
DisplayGraphics_draw_text(ptr_main_module->m_gfx,text, pos, y );
}

/*
Ordered dithering
*/



void ordered(char *out,char *in,int width,int yp)
{
long px=0,apos=0;
int x,y,i,j,c,col,aw;
char p1,p2,p3,p4;

char pattern[8][8] = {
    { 0, 32,  8, 40,  2, 34, 10, 42},   /* 8x8 Bayer ordered dithering  */
    {48, 16, 56, 24, 50, 18, 58, 26},   /* pattern.  Each input pixel   */
    {12, 44,  4, 36, 14, 46,  6, 38},   /* is scaled to the 0..63 range */
    {60, 28, 52, 20, 62, 30, 54, 22},   /* before looking in this table */
    { 3, 35, 11, 43,  1, 33,  9, 41},   /* to determine the action.     */
    {51, 19, 59, 27, 49, 17, 57, 25},
    {15, 47,  7, 39, 13, 45,  5, 37},
    {63, 31, 55, 23, 61, 29, 53, 21} };

j=0;
for(i=0;i<yp;i++)
{
j++;
if (j>7){j=0;}
}

// dither row
i=0;
for(x=0;x<width;x++)
{
col=in[x];
if (col<0){col+=256;}

c=(col & 0xff)>>2;
if (c<pattern[i][j]){in[x]=3-(c>>4);}else{in[x]=0;}


i++;
if (i>7){i=0;}

}
//

px=0;
aw=(width>>2);
apos=aw*(yp-1);

for(x=0;x<aw;x++)
	{
	p1=in[px++];
	p2=in[px++];
	p3=in[px++];
	p4=in[px++];

	out[apos++]=(p1<<6)+(p2<<4)+(p3<<2)+p4;
	}

}


void snapshot(char *in)
{
struct FileOutput file_output;
long hd1,hd2;
char *fn,i=0,exists=0;
fn=(char *)calloc(30,sizeof(char));

FileOutput_ctor( &file_output );

hd1=0x0201a064;
hd2=0x0000a064;

do
{
	exists=0;
	memset(fn,0,sizeof(fn));
	sprintf(fn,"stormshot%d.pic",i);
	if( File_exists( fn )) {exists=1;i++;}
}while(exists==1);

if( FileOutput_open( &file_output, fn, TRUE ) )
{
FileOutput_write( &file_output,&hd1,4);
FileOutput_write( &file_output,&hd2,4);
FileOutput_write( &file_output,in,4000);
}

FileOutput_dtor( &file_output, LEAVE_MEMORY );
TRACE("Snapshot Taken, saved as %s",fn);
free(fn);
}
